@@ -32,8 +32,10 @@ impl Handler for MyHandler {
 
 #[tokio::main(flavor = "current_thread")]
 async fn main() -> anyhow::Result<()> {
-    let server = Server::new()?;
-    let connection = server.accept().await?;
+    pretty_env_logger::init();
+
+    let mut server = Server::new()?;
+    let (connection, _) = server.accept().await?.unwrap();
     connection.run(MyHandler).await?;
     Ok(())
 }
